unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtDlgs, GR32,JPEG, ExtCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Filter1: TMenuItem;
    Laplace1: TMenuItem;
    Hipass1: TMenuItem;
    FindEdgestopdown1: TMenuItem;
    Sharpen1: TMenuItem;
    EdgeEnhance1: TMenuItem;
    ColorEmbossSorta1: TMenuItem;
    Soften1: TMenuItem;
    Blur1: TMenuItem;
    LAP11: TMenuItem;
    LAP41: TMenuItem;
    HP31: TMenuItem;
    FindEdgesBottomtop1: TMenuItem;
    Sobelhor1: TMenuItem;
    Sobelvert1: TMenuItem;
    Previthor1: TMenuItem;
    Previtvert1: TMenuItem;
    LaplaceOrigin1: TMenuItem;
    GaussianBlure1: TMenuItem;
    OriginReset1: TMenuItem;
    Filter5x51: TMenuItem;
    Soft1: TMenuItem;
    OpenPictureDialog: TOpenPictureDialog;
    About1: TMenuItem;
    Special1: TMenuItem;
    Kuwahara5x51: TMenuItem;
    Gray1: TMenuItem;
    R1: TMenuItem;
    G1: TMenuItem;
    B1: TMenuItem;
    Color1: TMenuItem;
    USMSD071: TMenuItem;
    NewTest: TMenuItem;
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    procedure DoFilterClick(Sender: TObject);
    procedure Filter55Click(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Kuwahara5x51Click(Sender: TObject);

  private
    { Private declarations }
    InBitmap     : TBitmap32;
    BackBuffer   : TBitmap32;
    procedure NewTestClick(Sender: TObject);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
Uses Pathes,G32_WConvolution,AboutUnit,pqConvolution;
{$R *.DFM}

//this is from GR32_Filters
const
  SEmptySource = 'The source is nil';
  SEmptyDestination = 'Destination is nil';
procedure CheckParams(Dst, Src: TBitmap32);
begin
  if Src = nil then raise Exception.Create(SEmptySource);
  if Dst = nil then raise Exception.Create(SEmptyDestination);
  Dst.SetSize(Src.Width, Src.Height);
end;
//\this is from GR32_Filters


procedure TForm1.Open1Click(Sender: TObject);
Var
 FileName : String;
begin
 OpenPictureDialog.InitialDir:=GetName('');
 if Not OpenPictureDialog.Execute then Exit;
 InBitmap.LoadFromFile(OpenPictureDialog.FileName);

 CheckParams(BackBuffer,InBitmap);
 BackBuffer.Assign(InBitmap);
 Invalidate;
end;


procedure TForm1.FormCreate(Sender: TObject);
var
  NewItem: TMenuItem;
  I : integer;
begin
 InBitmap:=TBitmap32.Create;
 BackBuffer:=TBitmap32.Create;

 { now create and add a menu item for each filter name }
 for i:=low(CONVOLUTION_FILTERNAMES) to high(CONVOLUTION_FILTERNAMES) do
  begin
    NewItem := TMenuItem.Create(Self);
    NewItem.Caption := CONVOLUTION_FILTERNAMES[i];
    NewItem.Tag:=i;
    NewItem.OnClick:=NewTestClick;
    NewTest.Add(NewItem);
  end;

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 InBitmap.Free;
 BackBuffer.Free;
end;


procedure TForm1.DoFilterClick(Sender: TObject);
var
  z   : integer;
  ray : array [0..8] of integer;
begin
Case  TMenuItem(Sender).tag of

    0 : begin // Laplace
      ray[0] := -1; ray[1] := -1; ray[2] := -1;
      ray[3] := -1; ray[4] :=  8; ray[5] := -1;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 1;
      end;

    1 : begin  // Hipass
      ray[0] := -1; ray[1] := -1; ray[2] := -1;
      ray[3] := -1; ray[4] :=  9; ray[5] := -1;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 1;
      end;
    2 : begin  // Find Edges (top down)
      ray[0] :=  1; ray[1] :=  1; ray[2] :=  1;
      ray[3] :=  1; ray[4] := -2; ray[5] :=  1;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 1;
      end;
    3 : begin  // Sharpen
      ray[0] := -1; ray[1] := -1; ray[2] := -1;
      ray[3] := -1; ray[4] := 16; ray[5] := -1;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 8;
      end;
    4 : begin  // Edge Enhance
      ray[0] :=  0; ray[1] := -1; ray[2] :=  0;
      ray[3] := -1; ray[4] :=  5; ray[5] := -1;
      ray[6] :=  0; ray[7] := -1; ray[8] :=  0;
      z := 1;
      end;
    5 : begin  // Color Emboss (Sorta)
      ray[0] :=  1; ray[1] :=  0; ray[2] :=  1;
      ray[3] :=  0; ray[4] :=  0; ray[5] :=  0;
      ray[6] :=  1; ray[7] :=  0; ray[8] := -2;
      z := 1;
      end;
    6 : begin  // Soften
      ray[0] :=  2; ray[1] :=  2; ray[2] :=  2;
      ray[3] :=  2; ray[4] :=  0; ray[5] :=  2;
      ray[6] :=  2; ray[7] :=  2; ray[8] :=  2;
      z := 16;
      end;
    7 : begin  // Blur
      ray[0] :=  3; ray[1] :=  3; ray[2] :=  3;
      ray[3] :=  3; ray[4] :=  8; ray[5] :=  3;
      ray[6] :=  3; ray[7] :=  3; ray[8] :=  3;
      z := 32;
      end;

    8 : begin  // LAP1
      ray[0] :=  0; ray[1] :=  1; ray[2] :=  0;
      ray[3] :=  1; ray[4] := -4; ray[5] :=  1;
      ray[6] :=  0; ray[7] :=  1; ray[8] :=  0;
      z := 1;
      end;

    9 : begin  // LAP4
      ray[0] :=  1; ray[1] := -2; ray[2] :=  1;
      ray[3] := -2; ray[4] :=  4; ray[5] := -2;
      ray[6] :=  1; ray[7] := -2; ray[8] :=  1;
      z := 1;
      end;

    10 : begin  // HP3
      ray[0] :=  1; ray[1] := -2; ray[2] :=  1;
      ray[3] := -2; ray[4] :=  5; ray[5] := -2;
      ray[6] :=  1; ray[7] := -2; ray[8] :=  1;
      z := 1;
      end;

    11 : begin  // Find Edges (Bottom top )
      ray[0] := -1; ray[1] := -1; ray[2] := -1;
      ray[3] :=  1; ray[4] := -2; ray[5] :=  1;
      ray[6] :=  1; ray[7] :=  1; ray[8] :=  1;
      z := 1;
      end;

    12 : begin  // Sobel hor
      ray[0] :=  1; ray[1] :=  2; ray[2] :=  1;
      ray[3] :=  0; ray[4] :=  0; ray[5] :=  0;
      ray[6] := -1; ray[7] := -2; ray[8] := -1;
      z := 1;
      end;

    13 : begin  // Sobel vert
      ray[0] :=  1; ray[1] :=  0; ray[2] := -1;
      ray[3] :=  2; ray[4] :=  0; ray[5] := -2;
      ray[6] :=  1; ray[7] :=  0; ray[8] := -1;
      z := 1;
      end;

    14 : begin  // Previt hor
      ray[0] :=  1; ray[1] :=  1; ray[2] :=  1;
      ray[3] :=  0; ray[4] :=  0; ray[5] :=  0;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 1;
      end;

    15 : begin  // Previt vert
      ray[0] :=  1; ray[1] :=  0; ray[2] := -1;
      ray[3] :=  1; ray[4] :=  0; ray[5] := -1;
      ray[6] :=  1; ray[7] :=  0; ray[8] := -1;
      z := 1;
      end;


    16 : begin // Laplace - Origin
      ray[0] := -1; ray[1] := -1; ray[2] := -1;
      ray[3] := -1; ray[4] :=  7; ray[5] := -1;
      ray[6] := -1; ray[7] := -1; ray[8] := -1;
      z := 1;
      end;

    17 : begin   //Gaussian Blure
      ray[0] :=  1; ray[1] :=  2; ray[2] :=  1;
      ray[3] :=  2; ray[4] :=  4; ray[5] :=  2;
      ray[6] :=  1; ray[7] :=  2; ray[8] :=  1;
      z := 16;
         end;


    100 : begin
          CheckParams(BackBuffer,InBitmap);
          BackBuffer.Assign(InBitmap);
          BackBuffer.DrawTo(canvas.handle,0,0);
          Exit;
         end;
  end;


 CheckParams(BackBuffer,InBitmap);
 BackBuffer.Assign(InBitmap);
 ConvolveI(ray,z,BackBuffer);
 BackBuffer.DrawTo(canvas.handle,0,0);

end;

procedure TForm1.Filter55Click(Sender: TObject);
var
  z   : integer;
  ray : array [0..24] of integer;
begin
Case  TMenuItem(Sender).tag of


    0 : begin // Soften
         ray[0]  := 1;  ray[1]  := 1;  ray[2]  := 1;  ray[3]  := 1;  ray[4]  := 1;
         ray[5]  := 1;  ray[6]  := 1;  ray[7]  := 1;  ray[8]  := 1;  ray[9]  := 1;
         ray[10] := 1;  ray[11] := 1;  ray[12] := 1;  ray[13] := 1;  ray[14] := 1;
         ray[15] := 1;  ray[16] := 1;  ray[17] := 1;  ray[18] := 1;  ray[19] := 1;
         ray[20] := 1;  ray[21] := 1;  ray[22] := 1;  ray[23] := 1;  ray[24] := 1;

         z := 25;
      end;


    1 : begin // USM   :  final=2*original minus SD=0.7 Gauss blur
         ray[0]  := 0;  ray[1]  := 0;  ray[2]  := -1;  ray[3]  := 0;  ray[4]  := 0;
         ray[5]  := 0;  ray[6]  := -8;  ray[7]  := -21;  ray[8]  := -8;  ray[9]  := 0;
         ray[10] := -1;  ray[11] := -21;  ray[12] := 299;  ray[13] := -21;  ray[14] := -1;
         ray[15] := 0;  ray[16] := -8;  ray[17] := -21;  ray[18] := -8;  ray[19] := 0;
         ray[20] := 0;  ray[21] := 0;  ray[22] := -1;  ray[23] := 0;  ray[24] := 0;

         z := 179;
      end;



  end;


 CheckParams(BackBuffer,InBitmap);
 BackBuffer.Assign(InBitmap);
 ConvolveI5x5(ray,z,BackBuffer);
 BackBuffer.DrawTo(canvas.handle,0,0);

end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 if InBitmap.Empty then Exit;

 if Not BackBuffer.Empty then
 begin
  BackBuffer.DrawTo(canvas.handle,0,0);
  Exit;
 end;

 InBitmap.DrawTo(canvas.handle,0,0)

end;

procedure TForm1.About1Click(Sender: TObject);
begin
 AboutForm:=TAboutForm.Create(Application);

 if AboutForm.ShowModal=mrOK then AboutForm.Free;

end;

procedure TForm1.Kuwahara5x51Click(Sender: TObject);
Var
ColorMode : TColorMode;
n         : integer;
begin
 CheckParams(BackBuffer,InBitmap);
 BackBuffer.Assign(InBitmap);

 n:=TMenuItem(Sender).Tag;
 ColorMode:=TColorMode(n);
 Kuwahara5x5(BackBuffer,ColorMode);

 BackBuffer.DrawTo(canvas.handle,0,0);
end;

procedure TForm1.NewTestClick(Sender: TObject);
begin
 CheckParams(BackBuffer,InBitmap);
 BackBuffer.Assign(InBitmap);
 //ConvolveI(ray,z,BackBuffer);
 ApplyConvolution( BackBuffer,KERNEL_ARRAY[TMenuItem(Sender).Tag]);

 BackBuffer.DrawTo(canvas.handle,0,0);
end;

end.
